#include <htc.h>
#include "serial.h"
#include "gps.h"

// GPS Data holder
struct gps_info_st gps_info;

// Init
void gps_init(void)
{
	ser_init();
}

// Set wgs84 measuring system
void gps_setwgs84(void)
{
	char *msg_wgs84 = "$PSRF106,21*0F\r\n";
	while(*msg_wgs84){
		while(ser_getstate() == SER_TXBUSY) continue;
		ser_putchar(*msg_wgs84++);
	}
}

#define TO_NUMERIC(x)		((x) - '0')
static char old_ch[2];
static unsigned char index1, index2, index3;

unsigned char daymonth[] = { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};

// GPS interrupt handler
void gps_isr(void)
{
	char ch;

	ser_isr();

	switch( ser_getstate() ){

	case SER_RXREADY:
		// Serial driver has data

		// Read a byte
		ch = ser_getchar(); 
		
		// Looking for GPGGA message.
		// String GGA is unique among NEMA format
		if( old_ch[0] == 'G' && old_ch[1] == 'G' && ch == 'A' ){
			// GPGGA message found
			index1 = 0;
		} else {
			if(index1 < 127) index1 ++;
		}
		switch(index1){

				// LATITUDE
		case 13:	// 10 deg
			gps_info.latitude.deg = (short)TO_NUMERIC(ch) * 10;
			break;
		case 14: //  1 deg
			gps_info.latitude.deg += (short)TO_NUMERIC(ch);
			break;
		case 15: // 10 min
			gps_info.latitude.min = (short)TO_NUMERIC(ch) * 10;
			break;
		case 16:	//  1 min
			gps_info.latitude.min += (short)TO_NUMERIC(ch);
			break;
		case 18:	// 0.1 min
			gps_info.latitude.submin = (short)TO_NUMERIC(ch)  * 1000;
			break;
		case 19:	// 0.01 min
			gps_info.latitude.submin += (short)TO_NUMERIC(ch) * 100;
			break;
		case 20:	// 0.001 min
			gps_info.latitude.submin += (short)TO_NUMERIC(ch) * 10;
			break;
		case 21:	// 0.0001 min
			gps_info.latitude.submin += (short)TO_NUMERIC(ch);
			break;
		case 23: // N or S
			if(ch == 'N') gps_info.latitude.polarity = 1;
			else gps_info.latitude.polarity = 0;
			break;

				// LONGITUDE
		case 25:	// 100 deg
			gps_info.longitude.deg = (short)TO_NUMERIC(ch) * 100;
			break;
		case 26: //  10 deg
			gps_info.longitude.deg += (short)TO_NUMERIC(ch) * 10;
			break;
		case 27: //   1deg
			gps_info.longitude.deg += (short)TO_NUMERIC(ch);
			break;
		case 28: //  10 min
			gps_info.longitude.min = (short)TO_NUMERIC(ch) * 10;
			break;
		case 29:	//   1min
			gps_info.longitude.min += (short)TO_NUMERIC(ch);
			break;
		case 31:	// 0.1 min
			gps_info.longitude.submin = (short)TO_NUMERIC(ch) * 1000;
			break;
		case 32:	// 0.01 min
			gps_info.longitude.submin += (short)TO_NUMERIC(ch) * 100;
			break;
		case 33:	// 0.001 min
			gps_info.longitude.submin += (short)TO_NUMERIC(ch) * 10;
			break;
		case 34:	// 0.0001 min
			gps_info.longitude.submin += (short)TO_NUMERIC(ch);
			break;
		case 36:	// W or E
			if(ch == 'E') gps_info.longitude.polarity = 1;
			else gps_info.longitude.polarity = 0;
			break;

		case 38:	// State of measuring
			gps_info.state = TO_NUMERIC(ch);
			break;
				
				// Number of SATs
		case 40: // 10 sat
			gps_info.numsat = TO_NUMERIC(ch) * 10;
			break;
		case 41: //  1 sat
			gps_info.numsat += TO_NUMERIC(ch);
			break;

				// HDOPl
		case 43:	// 10 Dop
			gps_info.hdop.meter = (short)TO_NUMERIC(ch) * 10;
			break;
		case 44:	// 1 Dop
			gps_info.hdop.meter += (short)TO_NUMERIC(ch);
			break;
		case 46:	// 0.1Dop
			gps_info.hdop.submeter = (short)TO_NUMERIC(ch);
			break;

				// ALTITUDE
		case 48:	// 10000 meter
			gps_info.altitude.meter = (short)TO_NUMERIC(ch) * 10000;
			break;
		case 49:	// 1000 meter
			gps_info.altitude.meter += (short)TO_NUMERIC(ch) * 1000;
			break;
		case 50:	// 100 meter
			gps_info.altitude.meter += (short)TO_NUMERIC(ch) * 100;
			break;
		case 51:	// 10 meter
			gps_info.altitude.meter += (short)TO_NUMERIC(ch) * 10;
			break;
		case 52:	// 1 meter
			gps_info.altitude.meter += (short)TO_NUMERIC(ch);
			break;
		case 54:	// 0.1 meter
			gps_info.altitude.submeter = (short)TO_NUMERIC(ch);
			gps_info.numupdate++;
			break;

		default:
			break;

		} // switch(index)

		// Looking for GPZDA message ( to get day,month,year)
		if( old_ch[0] == 'Z' && old_ch[1] == 'D' && ch == 'A' ){
			index2 = 0;
		} else {
			if(index2 < 127) index2 ++;
		}
		switch(index2){

				// TIME
		case 2:	// 10 hour
			gps_info.daytime.hour = TO_NUMERIC(ch) * 10;
			break;
		case 3:	//  1 hour
			gps_info.daytime.hour += TO_NUMERIC(ch);
			break;
		case 4:	// 10 min
			gps_info.daytime.min = TO_NUMERIC(ch) * 10;
			break;
		case 5:	//  1 min
			gps_info.daytime.min += TO_NUMERIC(ch);
			break;
		case 6:	// 10 sec
			gps_info.daytime.sec = TO_NUMERIC(ch) * 10;
			break;
		case 7:	//  1 sec
			gps_info.daytime.sec += TO_NUMERIC(ch);
			break;

				// DATE
		case 13:	// 10 day
			gps_info.daytime.day = TO_NUMERIC(ch) * 10;
			break;
		case 14:	//  1 day
			gps_info.daytime.day += TO_NUMERIC(ch);
			break;
		case 16:	// 10 month
			gps_info.daytime.mon = TO_NUMERIC(ch) * 10;
			break;
		case 17:	//  1 month
			gps_info.daytime.mon += TO_NUMERIC(ch);
			break;
		case 21:	// 10 year
			gps_info.daytime.year = TO_NUMERIC(ch) * 10;
			break;
		case 22:	//  1 year
			gps_info.daytime.year += TO_NUMERIC(ch);

			// Localtime correction
			gps_info.daytime.hour += 9;		// JST offset from UTC
			if(gps_info.daytime.hour > 23){
				gps_info.daytime.hour -= 24;
				gps_info.daytime.day += 1;
			}
			if(gps_info.daytime.day > daymonth[gps_info.daytime.mon]){
				gps_info.daytime.day = 1;
				gps_info.daytime.mon += 1;
			}
			// Some more handwork required

			break;
		default:
			break;
		}

		// Looking for GPVTG message (to get speed)
		if( old_ch[0] == 'V' && old_ch[1] == 'T' && ch == 'G' ){
			index3 = 0;
		} else {
			if(index3 < 127) index3 ++;
		}
		switch(index3){
		case 24:
			if(ch == '9'){
				// Speed changes to 9999.99 km/h when no gps recieved or
				// out of precision tolerance. Holds previous value under 
				// this condition.
				index3 = 127;
				break;
			}
			gps_info.speed.meter = (short)TO_NUMERIC(ch) * 1000;
			break;
		case 25:
			gps_info.speed.meter += (short)TO_NUMERIC(ch) * 100;
			break;
		case 26:
			gps_info.speed.meter += (short)TO_NUMERIC(ch) * 10;
			break;
		case 27:
			gps_info.speed.meter += (short)TO_NUMERIC(ch);
			break;
		case 29:
			gps_info.speed.submeter = (short)TO_NUMERIC(ch) * 10;
			break;
		case 30:
			gps_info.speed.submeter += (short)TO_NUMERIC(ch);
			break;
		default:
			break;
		}

		old_ch[0] = old_ch[1];
		old_ch[1] = ch;
		break;

	case SER_RXERR:
		gps_info.numerr++;
		ser_getchar();
		break;

	default:
		break;
	}
}