#include <htc.h>

/**************************************************************************
 * SPI layer functions
 * spi_init()
 * spi_trx()
 * spi_cs_assert()		macro
 * spi_cs_deassert()	macro
 **************************************************************************/

// Initialize with low clock
void spi_init(void)
{
	// Setup port direction
	TRISC7 = 0; 		// Use RC7 as SDO (OUT)
	// TRISB0 = 1;	// Use RB0 as SDI (IN)
	TRISB1 = 0; 		// Use RB1 as SCL (OUT)
	TRISB2 = 0;		// Use RB2 as CS  (OUT)

	// Use TMR2 to prescale spi clock
	T2CKPS0 = 0;
	T2CKPS1 = 0;
	PR2 = 40;
	TMR2ON = 1;

	SSPEN = 0;
	SMP = 1;		// Input data sampled at middle of data clock
	CKP = 1;		// Idle state for clock is h level
	CKE = 1;		// Transmit occurs on transition from Idle to active clock state
	SSPM3 = 0; 	// Master mode clock = TMR2 output
	SSPM2 = 0;
	SSPM1 = 1;
	SSPM0 = 1;
	SSPEN = 1;	// Must be set after all SPI configs are done.	
}

// Go full speed
void spi_fullspeed(void)
{
	SSPEN = 0;
	SSPM3 = 0;	// Set clock Fosc/4 (10MHz)
	SSPM2 = 0;
	SSPM1 = 0;
	SSPM0 = 0;
	SSPEN = 1;
}

// Transmit / Recv
unsigned char spi_trx(unsigned char tx_data)
{
	SSPBUF = tx_data;
	while(BF == 0) continue;
	return SSPBUF;
}
